@@ -1,3 +1,4 @@
+#![allow(clippy::integer_arithmetic)]
 //! This module defines memory regions
 
 use crate::{
@@ -23,7 +24,7 @@ pub struct MemoryRegion {
 }
 impl MemoryRegion {
     /// Creates a new MemoryRegion structure from a slice
-    pub fn new_from_slice(v: &[u8], vm_addr: u64, vm_gap_size: u64, is_writable: bool) -> Self {
+    pub fn new_from_slice(slice: &[u8], vm_addr: u64, vm_gap_size: u64, is_writable: bool) -> Self {
         let vm_gap_shift = if vm_gap_size > 0 {
             let vm_gap_shift =
                 std::mem::size_of::<u64>() as u8 * 8 - vm_gap_size.leading_zeros() as u8 - 1;
@@ -33,9 +34,9 @@ impl MemoryRegion {
             std::mem::size_of::<u64>() as u8 * 8 - 1
         };
         MemoryRegion {
-            host_addr: v.as_ptr() as u64,
+            host_addr: slice.as_ptr() as u64,
             vm_addr,
-            len: v.len() as u64,
+            len: slice.len() as u64,
             vm_gap_shift,
             is_writable,
         }
@@ -48,13 +49,13 @@ impl MemoryRegion {
         vm_addr: u64,
         len: u64,
     ) -> Result<u64, EbpfError<E>> {
-        let mut begin_offset = vm_addr - self.vm_addr;
+        let begin_offset = vm_addr - self.vm_addr;
         let is_in_gap = ((begin_offset >> self.vm_gap_shift as u32) & 1) == 1;
-        let gap_mask = (1 << self.vm_gap_shift) - 1;
-        begin_offset = (begin_offset & !gap_mask) >> 1 | (begin_offset & gap_mask);
-        if let Some(end_offset) = begin_offset.checked_add(len as u64) {
+        let gap_mask = (-1i64 << self.vm_gap_shift) as u64;
+        let gapped_offset = (begin_offset & gap_mask) >> 1 | (begin_offset & !gap_mask);
+        if let Some(end_offset) = gapped_offset.checked_add(len as u64) {
             if end_offset <= self.len && !is_in_gap {
-                return Ok(self.host_addr + begin_offset);
+                return Ok(self.host_addr + gapped_offset);
             }
         }
         Err(EbpfError::InvalidVirtualAddress(vm_addr))
@@ -85,7 +86,7 @@ impl std::cmp::Ord for MemoryRegion {
 }
 
 /// Type of memory access
-#[derive(Clone, PartialEq, Eq, Debug)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub enum AccessType {
     /// Read
     Load,
@@ -95,49 +96,31 @@ pub enum AccessType {
 
 /// Indirection to use instead of a slice to make handling easier
 pub struct MemoryMapping<'a> {
-    /// Mapped (valid) regions
+    /// Mapped memory regions
     regions: Box<[MemoryRegion]>,
-    /// Copy of the regions vm_addr fields to improve cache density
-    dense_keys: Box<[u64]>,
     /// VM configuration
     config: &'a Config,
 }
 impl<'a> MemoryMapping<'a> {
-    fn construct_eytzinger_order(
-        &mut self,
-        ascending_regions: &[MemoryRegion],
-        mut in_index: usize,
-        out_index: usize,
-    ) -> usize {
-        if out_index >= self.regions.len() {
-            return in_index;
-        }
-        in_index = self.construct_eytzinger_order(ascending_regions, in_index, 2 * out_index + 1);
-        self.regions[out_index] = ascending_regions[in_index].clone();
-        self.dense_keys[out_index] = ascending_regions[in_index].vm_addr;
-        self.construct_eytzinger_order(ascending_regions, in_index + 1, 2 * out_index + 2)
-    }
-
     /// Creates a new MemoryMapping structure from the given regions
     pub fn new<E: UserDefinedError>(
         mut regions: Vec<MemoryRegion>,
         config: &'a Config,
     ) -> Result<Self, EbpfError<E>> {
-        let mut result = Self {
-            regions: vec![MemoryRegion::default(); regions.len()].into_boxed_slice(),
-            dense_keys: vec![0; regions.len()].into_boxed_slice(),
-            config,
-        };
         regions.sort();
-        for index in 1..regions.len() {
-            let first = &regions[index - 1];
-            let second = &regions[index];
-            if first.vm_addr.saturating_add(first.len) > second.vm_addr {
-                return Err(EbpfError::VirtualAddressOverlap(second.vm_addr));
+        for (index, region) in regions.iter().enumerate() {
+            if region.vm_addr != (index as u64) << ebpf::VIRTUAL_ADDRESS_BITS
+                || (region.len > 0
+                    && ((region.vm_addr + region.len - 1) >> ebpf::VIRTUAL_ADDRESS_BITS) as usize
+                        != index)
+            {
+                return Err(EbpfError::InvalidMemoryRegion(index));
             }
         }
-        result.construct_eytzinger_order(&regions, 0, 0);
-        Ok(result)
+        Ok(Self {
+            regions: regions.into_boxed_slice(),
+            config,
+        })
     }
 
     /// Given a list of regions translate from virtual machine to host address
@@ -147,57 +130,35 @@ impl<'a> MemoryMapping<'a> {
         vm_addr: u64,
         len: u64,
     ) -> Result<u64, EbpfError<E>> {
-        let mut index = 1;
-        while index <= self.dense_keys.len() {
-            index = (index << 1) + (self.dense_keys[index - 1] <= vm_addr) as usize;
-        }
-        index >>= index.trailing_zeros() + 1;
-        if index == 0 {
-            return Err(self.generate_access_violation(access_type, vm_addr, len));
-        }
-        let region = &self.regions[index - 1];
-        if access_type == AccessType::Load || region.is_writable {
-            if let Ok(host_addr) = region.vm_to_host::<E>(vm_addr, len as u64) {
-                return Ok(host_addr);
+        let index = (vm_addr >> ebpf::VIRTUAL_ADDRESS_BITS) as usize;
+        if (1..self.regions.len()).contains(&index) {
+            let region = &self.regions[index];
+            if access_type == AccessType::Load || region.is_writable {
+                if let Ok(host_addr) = region.vm_to_host::<E>(vm_addr, len as u64) {
+                    return Ok(host_addr);
+                }
             }
         }
-        Err(self.generate_access_violation(access_type, vm_addr, len))
-    }
-
-    /// Resize the memory_region at the given index
-    pub fn resize_region<E: UserDefinedError>(
-        &mut self,
-        index: usize,
-        new_len: u64,
-    ) -> Result<(), EbpfError<E>> {
-        if index < self.regions.len() - 1
-            && self.regions[index].vm_addr.saturating_add(new_len) > self.regions[index + 1].vm_addr
-        {
-            return Err(EbpfError::VirtualAddressOverlap(
-                self.regions[index + 1].vm_addr,
-            ));
-        }
-        self.regions[index].len = new_len;
-        Ok(())
+        self.generate_access_violation(access_type, vm_addr, len)
     }
 
     /// Helper for map to generate errors
-    fn generate_access_violation<E: UserDefinedError>(
+    pub fn generate_access_violation<E: UserDefinedError>(
         &self,
         access_type: AccessType,
         vm_addr: u64,
         len: u64,
-    ) -> EbpfError<E> {
+    ) -> Result<u64, EbpfError<E>> {
         let stack_frame =
             (vm_addr as i64 - ebpf::MM_STACK_START as i64) / self.config.stack_frame_size as i64;
         if (-1..self.config.max_call_depth as i64 + 1).contains(&stack_frame) {
-            EbpfError::StackAccessViolation(
+            Err(EbpfError::StackAccessViolation(
                 0, // Filled out later
                 access_type,
                 vm_addr,
                 len,
                 stack_frame,
-            )
+            ))
         } else {
             let region_name = match vm_addr & !(ebpf::MM_PROGRAM_START - 1) {
                 ebpf::MM_PROGRAM_START => "program",
@@ -206,13 +167,31 @@ impl<'a> MemoryMapping<'a> {
                 ebpf::MM_INPUT_START => "input",
                 _ => "unknown",
             };
-            EbpfError::AccessViolation(
+            Err(EbpfError::AccessViolation(
                 0, // Filled out later
                 access_type,
                 vm_addr,
                 len,
                 region_name,
-            )
+            ))
         }
     }
+
+    /// Resize the memory_region at the given index
+    pub fn resize_region<E: UserDefinedError>(
+        &mut self,
+        index: usize,
+        new_len: u64,
+    ) -> Result<(), EbpfError<E>> {
+        if index >= self.regions.len()
+            || (new_len > 0
+                && ((self.regions[index].vm_addr + new_len - 1) >> ebpf::VIRTUAL_ADDRESS_BITS)
+                    as usize
+                    != index)
+        {
+            return Err(EbpfError::InvalidMemoryRegion(index));
+        }
+        self.regions[index].len = new_len;
+        Ok(())
+    }
 }
